class GoogleCaptcha(Exception):
    def __init__(self, *args):
        self.data = "Captcha detected"
        if args:
            self.data = args[0]
    def  __str__(self):
        return "GoogleCaptcha, {0}".format(self.data)

class GoogleCookiePolicies(Exception):
    def __init__(self, *args):
        self.data = "Cookie Policy detected"
        if args:
            self.data = args[0]
    def  __str__(self):
        return "GoogleCookiePolicies, {0}".format(self.data)

class BaiduDetection(Exception):
    def __init__(self, *args):
        self.data = "Robot detected"
        if args:
            self.data = args[0]
    def  __str__(self):
        return "BaiduDetection, {0}".format(self.data)

class YandexDetection(Exception):
    def __init__(self, *args):
        self.data = "Robo detected"
        if args:
            self.data = args[0]
    def  __str__(self):
        return "YandexDetection, {0}".format(self.data)


from random import randint
from emailfinder.utils.agent import user_agent
from emailfinder.utils.file.email_parser import get_emails
import requests


def search(target, total=275):
	bing_count = 25
	emails = set()
	url = f"https://www.bing.com/search?q=inbody:'@{target}'&count={bing_count}"
	try:
		count = 0
		iter_count = int(total/bing_count)
		if (total%bing_count) != 0:
			iter_count +=1
		while count < iter_count:
			this_count = count*bing_count + 1
			new_url = url + f"&first={this_count}&FORM=PERE"
			response = requests.get(new_url, 
			headers=user_agent.get(randint(0, len(user_agent)-1)),
			timeout=5,
			verify=False)
			text = response.text
			emails = emails.union(get_emails(target, text))
			count += 1
	except:
		pass
	return list(emails)	



import requests
from time import sleep
from random import randint
from emailfinder.utils.exception import BaiduDetection
from emailfinder.utils.agent import user_agent
from emailfinder.utils.file.email_parser import get_emails
import urllib3
urllib3.disable_warnings()


def search(target, total=100):
	old_text = ""
	num_results = 50 if total >= 50 else total
	emails = set()
	base_url = "https://www.baidu.com/s?ie=utf-8"
	total_loop = int(total/num_results)
	if (total%num_results) != 0:
		total_loop += 1
	count = 1
	old_useragent = -1
	total_timeout = 0
	while count <= total_loop:
		while True:
			next_useragent = randint(0, len(user_agent)-1)
			if next_useragent != old_useragent:
				break
		old_useragent = next_useragent
		new_url = base_url + f'&pn={count*num_results}&wd=inbody:"@{target}"&rn={num_results}'
		try:
			new_agent = user_agent.get(count, next_useragent)
			response = requests.get(new_url, headers=new_agent, timeout=5, verify=False)
			text = response.text
			if old_text == text:
				break
			old_text = text
			if "timeout-button" in text:
				total_timeout += 1
				if total_timeout == 5:
					raise BaiduDetection
				sleep(2)
				continue
			emails = emails.union(get_emails(target, text))
		except Exception as ex:
			raise ex #It's left over... but it stays there
		count += 1
	return list(emails)	



user_agent = {
		0: {'User-agent': 'Mozilla/5.0 (Linux; Android 10; SM-A205U) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.86 Mobile Safari/537.36'},
		1: {'User-agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko'},
		2: {'User-agent': 'Opera/9.80 (Linux armv7l) Presto/2.12.407 Version/12.51 , D50u-D1-UHD/V1.5.16-UHD (Vizio, D50u-D1, Wireless)'},
		3: {'User-agent': 'BrightSign/7.1.95 (XT1143) Mozilla/5.0 (Unknown; Linux arm) AppleWebKit/537.36 (KHTML, like Gecko) QtWebEngine/5.6.0 Chrome/45.0.2454.101 Safari/537.36'},
		4: {'User-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.24 Safari/537.36'},
		5: {'User-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.150 Safari/537.36'},
		6: {'User-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.157 Safari/537.36'}
	}


import re


def get_emails(target, text):
    regex = r"[a-zA-Z0-9_\.+-]+@" + target
    emails = re.findall(regex, text.replace("<em>", "").replace("<\em>","")
                                .replace("<strong>", "").replace("</strong>", "")
                                .replace("<b>", "").replace("</b>", ""))
    return emails




import requests
from random import randint
from emailfinder.utils.exception import GoogleCaptcha, GoogleCookiePolicies
from emailfinder.utils.agent import user_agent
from emailfinder.utils.file.email_parser import get_emails
import urllib3
urllib3.disable_warnings()


def search(target, proxies=None, total=200):
	emails = set()
	start = 0
	num = 50 if total > 50 else total
	iterations = int(total/50)
	if (total%50) != 0:
		iterations += 1
	url_base = f"https://www.google.com/search?q=intext:@{target}&num={num}"
	cookies = {"CONSENT": "YES+srp.gws"}
	while start < iterations:
		try:
			url = url_base + f"&start={start}"
			response = requests.get(url, 
			# headers={'User-agent': 'APIs-Google (+https://developers.google.com/webmasters/APIs-Google.html)'},
			headers=user_agent.get(randint(0, len(user_agent)-1)),
			verify=False,
			allow_redirects=False,
			cookies=cookies)
			text = response.text
			if response.status_code == 302 and ("htps://www.google.com/webhp" in text or "https://consent.google.com" in text):
				raise GoogleCookiePolicies()
			elif "detected unusual traffic" in text:
				raise GoogleCaptcha()
			emails = emails.union(get_emails(target, text))
		except Exception as ex:
			raise ex #It's left over... but it stays there
		start += 1
	return list(emails)	


from emailfinder.utils.finder import google
from emailfinder.utils.finder import bing
from emailfinder.utils.finder import baidu
from emailfinder.utils.finder import yandex
from emailfinder.utils.color_print import print_error, print_ok
from concurrent.futures import ThreadPoolExecutor, as_completed


SEARCH_ENGINES_METHODS = {
        "google": google.search,
        "bing": bing.search,
        "baidu": baidu.search,
        "yandex": yandex.search
}


def _search(engine, target):
    emails = None
    print(f"Searching in {engine}...")
    try:
        emails = SEARCH_ENGINES_METHODS[engine](target)
        print_ok(f"{engine} done!")
    except Exception as ex:
        print_error(f"{engine} error {ex}")
    return emails
  
def _get_emails(target):
    threads = 4
    emails = set()
    with ThreadPoolExecutor(max_workers=threads) as executor:
        future_emails = {executor.submit(_search, engine, target): engine for engine in SEARCH_ENGINES_METHODS.keys()}
        for future in as_completed(future_emails):
            try:
                data = future.result()
                if data:
                    emails = emails.union(data)
            except Exception as ex:
                print_error(f"Error: {ex}")
    return list(emails)

def processing(target):
    emails = _get_emails(target)
    total_emails = len(emails)
    emails_msg = f"\nTotal emails: {total_emails}"
    print(emails_msg)
    print("-" * len(emails_msg))
    if total_emails > 0:
        for email in emails:
            print(email)
    else:
        print("0 emails :(. Closing...")




from setuptools import setup, find_packages
from os import path
import emailfinder


long_description = """
**EmailFinder - Search emails through Search Engines**
======================================================

Installation:
-------------

    > pip3 install emailfinder

Upgrades are also available using:

    > pip3 install emailfinder --upgrade

Search Engines
--------------

- google: Ok (note cookies policy and Captcha!).
- bing: OK.
- baidu: OK (few requests).
- bing: Hunting Robots very fast.

Usage
-----

EmailFinder can be used in 2 ways:

CLI
---

    emailfinder -d domain.com

Parameters: 

- d: Specifies the target domain.
- v: Show EmailFinder version.


In Code
-------

from emailfinder.extractor import *


emails1 = get_emails_from_google("domain.com")
emails2 = get_emails_from_bing("domain.com")
emails3 = get_emails_from_baidu("domain.com")


Author
======

This project has been developed by:

-  **Josué Encinar García** -- https://twitter.com/JosueEncinar


Disclaimer!
===========

The software is designed to leave no trace in the documents we upload to a domain. The author is not responsible for any
illegitimate use.



"""

setup(
    name='emailfinder',
    version=emailfinder.__version__,
    author='Josue Encinar (@JosueEncinar)',
    description='EmailFinder - Emails search through Search Engines',
    include_package_data=True,
    license='GNU GPLv3+',
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/Josue87/EmailFinder",
    entry_points={
        'console_scripts': [
            'emailfinder=emailfinder.cli:main',
        ],
    },
    install_requires=[
        "requests>=2.25.1",
        "prompt-toolkit>=3.0.5",
        "urllib3>=1.26.4"
    ]
)



from emailfinder.utils.finder import google
from emailfinder.utils.finder import baidu
from emailfinder.utils.finder import bing


def get_emails_from_google(target):
    return google.search(target)

def get_emails_from_bing(target):
    return bing.search(target)

def get_emails_from_baidu(target):
    return baidu.search(target)